# Copyright 2018 Google Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file in README.md and
#     CONTRIBUTING.md located at the root of this package.
#
# ----------------------------------------------------------------------------

require 'google/storage/property/array'
require 'google/storage/property/base'

module Google
  module Storage
    module Data
      # A class to manage data for Cors for bucket.
      class BucketCors
        include Comparable

        attr_reader :max_age_seconds
        attr_reader :method
        attr_reader :origin
        attr_reader :response_header

        def to_json(_arg = nil)
          {
            'maxAgeSeconds' => max_age_seconds,
            'method' => method,
            'origin' => origin,
            'responseHeader' => response_header
          }.reject { |_k, v| v.nil? }.to_json
        end

        def to_s
          {
            max_age_seconds: max_age_seconds,
            method: method,
            origin: origin,
            response_header: response_header
          }.reject { |_k, v| v.nil? }.map { |k, v| "#{k}: #{v}" }.join(', ')
        end

        def ==(other)
          return false unless other.is_a? BucketCors
          compare_fields(other).each do |compare|
            next if compare[:self].nil? || compare[:other].nil?
            return false if compare[:self] != compare[:other]
          end
          true
        end

        def <=>(other)
          return false unless other.is_a? BucketCors
          compare_fields(other).each do |compare|
            next if compare[:self].nil? || compare[:other].nil?
            result = compare[:self] <=> compare[:other]
            return result unless result.zero?
          end
          0
        end

        private

        def compare_fields(other)
          [
            { self: max_age_seconds, other: other.max_age_seconds },
            { self: method, other: other.method },
            { self: origin, other: other.origin },
            { self: response_header, other: other.response_header }
          ]
        end
      end

      # Manages a BucketCors nested object
      # Data is coming from the GCP API
      class BucketCorsApi < BucketCors
        def initialize(args)
          @max_age_seconds = Google::Storage::Property::Integer.api_munge(args['maxAgeSeconds'])
          @method = Google::Storage::Property::StringArray.api_munge(args['method'])
          @origin = Google::Storage::Property::StringArray.api_munge(args['origin'])
          @response_header =
            Google::Storage::Property::StringArray.api_munge(args['responseHeader'])
        end
      end

      # Manages a BucketCors nested object
      # Data is coming from the Puppet manifest
      class BucketCorsCatalog < BucketCors
        def initialize(args)
          @max_age_seconds =
            Google::Storage::Property::Integer.unsafe_munge(args['max_age_seconds'])
          @method = Google::Storage::Property::StringArray.unsafe_munge(args['method'])
          @origin = Google::Storage::Property::StringArray.unsafe_munge(args['origin'])
          @response_header =
            Google::Storage::Property::StringArray.unsafe_munge(args['response_header'])
        end
      end
    end

    module Property
      # A class to manage input to Cors for bucket.
      class BucketCors < Google::Storage::Property::Base
        # Used for parsing Puppet catalog
        def unsafe_munge(value)
          self.class.unsafe_munge(value)
        end

        # Used for parsing Puppet catalog
        def self.unsafe_munge(value)
          return if value.nil?
          Data::BucketCorsCatalog.new(value)
        end

        # Used for parsing GCP API responses
        def self.api_munge(value)
          return if value.nil?
          Data::BucketCorsApi.new(value)
        end
      end

      # A Puppet property that holds an integer
      class BucketCorsArray < Google::Storage::Property::Array
        # Used for parsing Puppet catalog
        def unsafe_munge(value)
          self.class.unsafe_munge(value)
        end

        # Used for parsing Puppet catalog
        def self.unsafe_munge(value)
          return if value.nil?
          return BucketCors.unsafe_munge(value) \
            unless value.is_a?(::Array)
          value.map { |v| BucketCors.unsafe_munge(v) }
        end

        # Used for parsing GCP API responses
        def self.api_munge(value)
          return if value.nil?
          return BucketCors.api_munge(value) \
            unless value.is_a?(::Array)
          value.map { |v| BucketCors.api_munge(v) }
        end
      end
    end
  end
end
